@@ -1,11 +1,12 @@
 //! MRT table dump version 1 and 2 structs
-use std::net::IpAddr;
+use std::net::{IpAddr, Ipv4Addr};
 use std::collections::HashMap;
-use crate::bgp::attributes::Attributes;
 use crate::network::{Afi, Asn, NetworkPrefix, Safi};
+use serde::Serialize;
+use crate::bgp::Attribute;
 
 /// TableDump message version 1
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct TableDumpMessage {
     pub view_number: u16,
     pub sequence_number: u16,
@@ -14,11 +15,11 @@ pub struct TableDumpMessage {
     pub originated_time: u64,
     pub peer_address: IpAddr,
     pub peer_asn: Asn,
-    pub attributes: Attributes,
+    pub attributes: Vec<Attribute>,
 }
 
 /// TableDump message version 2 enum
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub enum TableDumpV2Message {
     PeerIndexTable(PeerIndexTable),
     RibAfiEntries(RibAfiEntries),
@@ -28,7 +29,7 @@ pub enum TableDumpV2Message {
 /// TableDump version 2 subtypes.
 ///
 /// <https://www.iana.org/assignments/mrt/mrt.xhtml#subtype-codes>
-#[derive(Debug, Primitive, Copy, Clone)]
+#[derive(Debug, Primitive, Copy, Clone, Serialize, PartialEq, Eq)]
 pub enum TableDumpV2Type{
     PeerIndexTable = 1,
     RibIpv4Unicast = 2,
@@ -73,8 +74,9 @@ pub enum TableDumpV2Type{
 ///        |         Entry Count           |  RIB Entries (variable)
 ///        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct RibAfiEntries{
+    pub rib_type: TableDumpV2Type,
     pub sequence_number: u32,
     pub prefix: NetworkPrefix,
     pub rib_entries: Vec<RibEntry>,
@@ -101,7 +103,7 @@ pub struct RibAfiEntries{
 ///        |         Entry Count           |  RIB Entries (variable)
 ///        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct RibGenericEntries{
     pub sequence_number: u32,
     pub afi: Afi,
@@ -131,11 +133,11 @@ pub struct RibGenericEntries{
 ///        |                    BGP Attributes... (variable)
 ///        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct RibEntry {
     pub peer_index: u16,
     pub originated_time: u32,
-    pub attributes: Attributes
+    pub attributes: Vec<Attribute>
 }
 
 /// peer index table.
@@ -149,9 +151,9 @@ pub struct RibEntry {
 ///    itself and includes full MRT record headers.  The RIB entry MRT
 ///    records MUST immediately follow the PEER_INDEX_TABLE MRT record.
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct PeerIndexTable{
-    pub collector_bgp_id: u32,
+    pub collector_bgp_id: Ipv4Addr,
     pub view_name_length: u16,
     pub view_name: String,
     pub peer_count: u16,
@@ -159,10 +161,10 @@ pub struct PeerIndexTable{
 }
 
 /// Peer struct.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct Peer {
     pub peer_type: u8,
-    pub peer_bgp_id: u32,
+    pub peer_bgp_id: Ipv4Addr,
     pub peer_address: IpAddr,
     pub peer_asn: Asn,
 }
\ No newline at end of file
